function mean_Y=estim_auxil(Dataset,prior_intrcp,c1,c,N,P,thita,num_iter,num_burn,num_thin,varargin)

%ESTIM_AUXIL   This M-File calculates the mean of the auxiliary variable
%              introduced by Albert and Chip (1993) and is roughly estimated by running few iterations of the
%              Holmes and Held algorithm.

target=Dataset(:,1);
X=Dataset;X(:,1)=[];

[n,p]=size(X);

%The probability that each explanatory variable is included in the model.
thita=thita/p;

%Initial model
model=binornd(1,thita,1,p);
l=find(model);
k=length(l);
t=ones(n,1);

%Model Probability
prob=k*log(thita)+(p-k)*log(1-thita);


%Initial values of the auxiliary variable.
Y(find(target==1),1)=2;Y(find(target==0),1)=-2; % Initial value of Y.

%Log-likelihood of the model and number of predictors.
num=(num_iter-num_burn)/num_thin;
auxiliary=sparse(num,n);

%Initial value of beta.
if isequal(prior_intrcp,'Proper')

    data=[t X(:,l)];
    H=diag([c1 c*ones(1,k)]);
    K=data'*data+diag([1/c1 1/c*ones(1,k)]);%Precision matrix.
    var=inv(K);%Covariance matrix.

    mu=var*data'*Y;
    %beta=mvnrnd(mu,var)';% Initial sample  of regression coefficients.
    beta=mu+chol(var)'*randn(k+1,1);


else
    %For improper prior.
    data=X(:,l);
    H=diag(c*ones(1,k));
    K=data'*data+diag(1/c*ones(1,k));%Precision matrix.
    var=inv(K);%Covariance matrix.

    mu=var*data'*Y;
    beta=mu+chol(var)'*randn(k,1);
    intrcp=mean(Y)+1/(sqrt(n))*randn;
    beta=[intrcp;beta];

end



%Start the Reversible Jump McMc iteration.

for i=1:num_iter


    %Propose new gamma.

    proposemodel=model;
    w=1+binornd(N,P); %Block size.
    bin_prob=binopdf(w-1,N,P);

    if k<w
        move=2;
    elseif k==w
        move=1+unidrnd(2);
    else
        move=unidrnd(3);
    end

    if move==1
        %Deletion move

        test=0;

        while test~=w %Select w dinstinct predictors.
            r=unidrnd(k,1,w);
            test=length(unique(r));
        end

        proposemodel(l(r))=0;
        newl=find(proposemodel);
        newk=k-w;
        newprob=newk*log(thita)+(p-newk)*log(1-thita);

        %Probability to move from model to proposemodel.
        r12=1/(3*combination(k,w))*bin_prob;

        %Probability to move from proposemodel to model.
        if newk==w

            r21=1/(2*combination(p-(k-w),w))*bin_prob;

        elseif newk<w

            r21=1/(combination(p-(k-w),w))*bin_prob;

        else

            r21=1/(3*combination(p-(k-w),w))*bin_prob;

        end

        if isequal(prior_intrcp,'Proper')

            newdata=[t X(:,newl)];
            newH=diag([c1 c*ones(1,newk)]);
            K1=newdata'*newdata+diag([1/c1 1/c*ones(1,newk)]);%Precision matrix.
            newvar=inv(K1);%Covariance matrix.

        else
            %For improper prior.
            newdata=X(:,newl);
            newH=diag(c*ones(1,newk));
            K1=newdata'*newdata+diag(1/c*ones(1,newk));%Precision matrix.
            newvar=inv(K1);%Covariance matrix.

        end

        Qmodel=Y'*(data*var*data')*Y;
        Qpropose=Y'*(newdata*newvar*newdata')*Y;

        accept=newprob-prob+0.5*(log(det(eye(n)+data*H*data'))-log(det(eye(n)+newdata*newH*newdata')))+0.5*(Qpropose-Qmodel)+log(r21)-log(r12);

    elseif move==2

        %Adittion move

        covariate=find(model==0);
        k1=length(covariate);
        test=0;

        while test~=w %Select w dinstinct predictors.
            r=unidrnd(k1,1,w);
            test=length(unique(r));
        end

        proposemodel(covariate(r))=1;

        newl=find(proposemodel);
        newk=k+w;
        newprob=newk*log(thita)+(p-newk)*log(1-thita);

        if k<w
            %Probability to move from model to proposemodel.
            r12 = 1/(combination(p-k,w))*bin_prob;


        elseif k==w

            %Proobability to move from model to proposemodel.
            r12 = 1/(2*combination(p-k,w))*bin_prob;

        else
            %Probability to move from model to proposemodel.
            r12 = 1/(3*combination(p-k,w))*bin_prob;

        end

        %Probability to move from proposemodel to model.
        r21=1/(3*combination(k+w,w))*bin_prob;

        if isequal(prior_intrcp,'Proper')

            newdata=[t X(:,newl)];
            newH=diag([c1 c*ones(1,newk)]);
            K1=newdata'*newdata+diag([1/c1 1/c*ones(1,newk)]);%Precision matrix.
            newvar=inv(K1);%Covariance matrix.

        else
            %For improper prior.
            newdata=X(:,newl);
            newH=diag(c*ones(1,newk));
            K1=newdata'*newdata+diag(1/c*ones(1,newk));%Precision matrix.
            newvar=inv(K1);%Covariance matrix.

        end

        Qmodel=Y'*(data*var*data')*Y;
        Qpropose=Y'*(newdata*newvar*newdata')*Y;

        accept=newprob-prob+0.5*(log(det(eye(n)+data*H*data'))-log(det(eye(n)+newdata*newH*newdata')))+0.5*(Qpropose-Qmodel)+log(r21)-log(r12);
    else
        %Swaption move
        test=0;

        while test~=w %Select w dinstinct predictors.
            r=unidrnd(k,1,w);
            test=length(unique(r));
        end

        proposemodel(l(r))=0;

        covariate=find(model==0);
        k1=length(covariate);
        test=0;

        while test~=w %Select w dinstinct predictors.
            rr=unidrnd(k1,1,w);
            test=length(unique(rr));
        end


        proposemodel(covariate(rr))=1;

        newl=find(proposemodel);
        newk=k;
        newprob=prob;

        if isequal(prior_intrcp,'Proper')

            newdata=[t X(:,newl)];
            newH=diag([c1 c*ones(1,newk)]);
            K1=newdata'*newdata+diag([1/c1 1/c*ones(1,newk)]);%Precision matrix.
            newvar=inv(K1);%Covariance matrix.

        else
            %For improper prior.
            newdata=X(:,newl);
            newH=diag(c*ones(1,newk));
            K1=newdata'*newdata+diag(1/c*ones(1,newk));%Precision matrix.
            newvar=inv(K1);%Covariance matrix.

        end

        Qmodel=Y'*(data*var*data')*Y;
        Qpropose=Y'*(newdata*newvar*newdata')*Y;

        accept=+0.5*(log(det(eye(n)+data*H*data'))-log(det(eye(n)+newdata*newH*newdata')))+0.5*(Qpropose-Qmodel);

    end

    accept=exp(accept);
    u=rand;

    if u<=accept

        model=proposemodel;
        data=newdata;
        prob=newprob;
        k=newk;
        l=newl;

        H=newH;
        var=newvar;%Covariance matrix.



        %Simulate the new betas from its full conditional given a
        %change to the covariate set.
        if isequal(prior_intrcp,'Proper')

            mu=var*data'*Y;
            beta=mu+chol(var)'*randn(k+1,1);

        else

            %For improper prior.
            mu=var*data'*Y;
            beta=mu+chol(var)'*randn(k,1);
            intrcp=mean(Y)+1/(sqrt(n))*randn;
            beta=[intrcp;beta];

        end

    end

    %Impute  the new value of Y from its full conditional given the model
    %and its parameters.

    if isequal(prior_intrcp,'Proper')

        m=data*beta;

    else

        m=[t data]*beta;

    end

    for j=1:n
        if target(j)==1
            Y(j)=test_truncnorm1(m(j),1,0,+inf);
        else
            Y(j)=test_truncnorm1(m(j),1,-inf,0);
        end
    end

    %Within model move.
    if isequal(prior_intrcp,'Proper')

        mu=var*data'*Y;
        beta=mu+chol(var)'*randn(k+1,1);

    else

        %For improper prior.
        mu=var*data'*Y;
        beta=mu+chol(var)'*randn(k,1);
        intrcp=mean(Y)+1/(sqrt(n))*randn;
        beta=[intrcp;beta];

    end



    if i<num_burn,continue,end

    if i==num_burn

        auxiliary((i-num_burn)/num_thin+1,:)=Y';

    elseif (mod(i,num_thin)==0)

        auxiliary((i-num_burn)/num_thin+1,:)=Y';

    end

end

mean_Y=mean(auxiliary)';


