function tau=integautocorr_time(v,method,varargin)
%INTEGAUTOCORR  Estimate the integrated autocorrelation time of the MCMC
%               output using either the initial positive sequense estimator
%               or the initial monotone sequence introduced by
%               Charles J.Geyer "Practical Markov Chain Monte Carlo".


%Last Modification: 14 Sep 2008
%%%%%%%%%%%%%%%%
%INPUT variables
%%%%%%%%%%%%%%%%
%v : matrix
%    n x 1 time series data.
%Method : character
%        'Positive' use the initial positive sequence estimator.
%         Otherwise use the initial monotone sequence estimator.
%
%Optional Input:
%     nlags : matrix 1 x 1.
%     Positive, scalar integer indicating the number of lags of the ACF
%     to compute.
%
%%%%%%%%%%%%%%%%%
%OUTPUT variables
%%%%%%%%%%%%%%%%%
%tau : matrix 1 x 1
%     Estimator of the integrated autocorrelation time.
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


nlags=30;%Default number of lags.

if nargin==3
    nlags=varargin{1};
end


p=autocorr(v,nlags);

for m=1:(nlags/2)

    P(m)=p(2*m-1)+p(2*m);

end

k=1;

while (P(k)>0 )

    k=k+1;
    if (k==((nlags/2)+1)),break;end

end

if k==1

    tau=1;

else

    i=1;

    if k~=2

        while (P(i)>P(i+1) )
            i=i+1;

            if (i==(k-1)),break;end

        end

    end



    if isequal(method,'Positive')

        tau=1+2*sum(p(2:2*(k-1)));

    else

        tau=1+2*sum(p(2:2*i));

    end

end